/**
 * JdbcTool: Command line pain relief for JDBC databases.
 * Copyright (C) 2007, Quuxo Software.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package com.quuxo.jdbctool;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;

import gnu.getopt.Getopt;


/**
 * Command line program to execute SQL statements from a file.
 *
 * @author <a href="mailto:michael@quuxo.com">Michael Gratton</a>
 */
public class JdbcLoad {


    protected String[] driverNames = new String[] { "org.hsqldb.jdbcDriver" };
    protected Connection connection;
    protected char eol = System.getProperty("line.separator").charAt(0);
    protected int lineNumber = 1;
    
    
    public JdbcLoad(String url, String username, String password)
        throws SQLException {
        loadDrivers();
        this.connection = DriverManager.getConnection(url, username, password);
    }

    protected void loadDrivers() {
        for (String name: this.driverNames) {
            try {
                Class.forName(name);
            } catch (Exception e) {
                // XXX log me
            }
        }
    }

    public void close()
        throws SQLException {
        this.connection.close();
    }

    public void load(Reader sql)
        throws SQLException, IOException {
        StringBuffer line = new StringBuffer();
        Statement statement = this.connection.createStatement();
        int b;
        char quote = ' '; // treat the space as null or -1
        this.lineNumber = 1;
        while ((b = sql.read()) != -1) {
            char c = (char) b;
            line.append(c); 
            if (c == eol) {
                this.lineNumber++;
            } else if (c == '\'' || c == '"') {
                if (quote == ' ') {
                    quote = c;
                } else if (quote == c) {
                    quote = ' ';
                }
            } else if (c == ';' && quote == ' ') {
                execute(statement, line.toString());
                line = new StringBuffer();
            }
        }
        String last = line.toString();
        if (last.trim().length() != 0) {
            execute(statement, line.toString());
        }
    }

    protected void execute(Statement s, String q)
        throws SQLException {
        ResultSet results = s.executeQuery(q);
        exhumeWarnings(results);
    }

    protected void exhumeWarnings(ResultSet results)
        throws SQLException {
        SQLWarning w = results.getWarnings();
        while (w != null) {
            printLineWarning(w.toString());
            w = w.getNextWarning();
        }
        results.clearWarnings();
    }

    protected void printLineWarning(String message) {
        System.err.println("Warning, line " + this.lineNumber + ": " + message);
    }

    protected void printLineError(String message) {
        System.err.println("Error, line " + this.lineNumber + ": " + message);
    }

    protected static void printError(String message) {
        System.err.println(message);
    }

    protected static String promptUser(String message) {
        try {
            BufferedReader in =
                new BufferedReader(new InputStreamReader(System.in));
            System.err.println(message + ": ");
            return in.readLine();
        } catch (IOException ioe) {
            printError(ioe.getMessage());
            return null;
        }
    }

    public static void main(String[] argv) {
        String file = null;
        String password = null;
        String user = null;

        Getopt g = new Getopt("JdbcLoad", argv, "f:p:Pu:");
        int c;
        while ((c = g.getopt()) != -1) {
            switch(c) {
            case 'f':
                file = g.getOptarg();
                break;
            case 'p':
                password = g.getOptarg();
                break;
            case 'P':
                password = promptUser("Enter password");
                break;
            case 'u':
                user = g.getOptarg();
                break;
            default:
                printError("JdbcLoad: unknown option `" + c + "'");
                System.exit(-1);
            }
        }        

        int i = g.getOptind();
        if (i >= argv.length) {
            printError("No JDBC URL specified.");
            System.exit(-1);
        }
        String url = argv[i];

        BufferedReader in = null;
        try {
            if (file != null) {
                in = new BufferedReader(new FileReader(file));
            } else {
                in = new BufferedReader(new InputStreamReader(System.in));
            }
        } catch (IOException ioe) {
            printError("Unable to open SQL file: " + ioe);
            System.exit(-1);
        }

        JdbcLoad jl = null;
        try {
            jl = new JdbcLoad(url, user, password);
        } catch (SQLException sqle) {
            printError("Unable to connect to database: " + sqle);
            System.exit(-2);
        }

        try {
            jl.load(in);
        } catch (Exception e) {
            jl.printLineError(e.getMessage());
            System.exit(-3);
        }

        try {
            jl.close();
        } catch (SQLException sqle) {
            printError("Error closing connection: " + sqle);
            System.exit(-4);
        }

    }

}
